#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2009,2010,2011  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# (version 2) as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import errno

################################################################################
# Environment
################################################################################

def get_paths(var):
  """Get a list of paths from an environment variable."""
  paths = os.getenv(var)
  if paths:
    return paths.strip(os.pathsep).split(os.pathsep)
  else:
    return []



def require_path(var):
  """Get an environment variable path.

  Raises an error if the variable is not set.
  """
  path = os.getenv(var)
  if path:
    return path
  else:
    raise UnsetEnvVarError(var)



def require_paths(var):
  """Get a list of environment variable paths.

  Raises an error if the variable is not set.
  """
  paths = get_paths(var)
  if paths:
    return paths
  else:
    raise UnsetEnvVarError(var)



def is_executable(path):
  """Check if the path is executable by the running process."""
  return os.path.exists(path) and os.access(path, os.X_OK)



def which(cmd):
  """Emulate the "which" command.

  Returns None if the argument could not be found.
  """
  if not cmd:
    return None
  fpath, fname = os.path.split(cmd)
  # If the command contains a full path, check that it is executable.
  if fpath:
    cmd = os.path.abspath(cmd)
    if is_executable(cmd):
      return cmd
    else:
      return None
  # Otherwise look up the name on the system path.
  else:
    for p in get_paths("PATH"):
      fpath = os.path.join(p, fname)
      if is_executable(fpath):
        return fpath
    else:
      return None



def make_parent_dirs(path, mode=0o777, exist_ok=True):
  """Recursively create all parent directories of the given path."""
  try:
    os.makedirs(os.path.dirname(path), mode=mode, exist_ok=exist_ok)
  except OSError as e:
    if e.errno != errno.EEXIST:
      raise e
